%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author of code needs to edit this section.
% This code has been tested in Matlab 6.5 on RHEL 3.0 and Octave 2.1.50
% on RHEL 3.0.
%
% Data are extracted from file ./data/*.dat
% 
% **YOUR_NAME and Date**
%
% Author also needs to modify the characters LastName-YYYY.MM.DD in the
% directory name and _Template_.m in the .m file names in this directory.
% _Template_ should be replaced with the name of the place where the data
% were downloaded from, e.g., ISGI, DMI_WDC, NSSDC, etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Template for contributing data to the CISM_DX uniform time series data
% set.  In this example we transform the ISGI am and aa data set to the
% "standard form".  Standard form is on a 1-day, 1-hour, or 1-minute time
% grid with fill values inserted where not data exist.  Fill values should
% be inserted such that the time series starts on the first time interval of
% the first year in which data are available and ends on last time interval
% of the last time interval in which data are available.  Data with a given
% time label should be based on measurements taken in the the interval
% _after_ the label, i.e., the 1st, 25th, ... data point in a 1-hour time
% grid series should be based on data in the interval 00:00:00.0000 to
% 00:59:59.9999.
%
% Any flag value is allowed, since it will be inserted into the metadata,
% but we prefer 99999, then -99999, then 999999, etc.
%
% We will contact the data source and ask for their permission to include
% their data in the CISM_DX uniform time series data set.
%
% This .m file transforms the aa data from its original file format to a
% temporary .mat file with a single variable named Data.  This .mat file is
% operated on using make_Template_part2.m and put into an hdf file.
%
% To use in Octave or Matlab, put the directory CISM_DX/mfiles is in your
% path and execute make_Template_part1 on the Octave or Matlab command line.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;

% Load global variables
CISM_DX_GLOBALS;

SUB_DATE = 'YYYY.MM.DD';

% Put path variables near the top of your program!
BASE           = '/home/gehmeyr/';
PERLFILES_ROOT = [BASE,'CISM_DX-LastName-',SUB_DATE,'/perlfiles/'];
DATA_DIR       = [BASE,'CISM_DX_DATA-LastName-',SUB_DATE,...
	 	 '/measurements/original_data/ISGI/aa'];

TEMP_DIR  = './';
TEMP_FILE = 'ISGI_aa_1_hour.mat'; % Temporary file name to be loaded by 
				  % make_Template_part2.m

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Transforming original data file using a Perl script into to a file that
% can be quickly loaded by Octave or Matlab.  CISM_DX_PERL_PATH is a 
% string variable
% loaded in call to CISM_DX_GLOBALS.  It is /usr/local/perl on Linux
% systems.  
%
% For your submission re-set PERLFILES_ROOT to ./ so that all of your
% code resides in this directory.
cwd = pwd;
cd(DATA_DIR);
com = sprintf('%s %sima2matrix.pl',CISM_DX_PERL_PATH,PERLFILES_ROOT);
      system(com);
% Load this file      
X   = load('data_cat.matrix');
cd(cwd);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matrix X has format
% Column 1   = MOYR (e.g., 12004 = January, 2004)
% Column 2   = CODE (18=aa, 2=am)
% Column 3   = Number of days in month
% Column 4   = Ave in 1st 3-hour interval of month
% ...
% Column 248 = Ave in 248th( = 8*31) 3-hour interval of month
%
% If month has less than 31 days, -1 is used as fill.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MO      = floor(X(:,1)/10000);
YR      = round(X(:,1)-10000*fix(X(:,1)/10000));

% Remove duplicates (if needed) and sort by year (if not already) 
[jnk,I] = unique(100*YR+MO); 
MO      = MO(I);
X       = X(I,:);

% Check for missing years
if (max(diff(YR)) > 1)
  error('failed: year missing');
else
  fprintf('passed: no missing years\n');
end

aa   = X(:,4:size(X,2));
Nd   = X(:,3);           % Number of days in month

Year_o = YR(1);
Year_f = YR(end);

% Check for missing months
if (length(YR) != 12*(Year_f - Year_o + 1))
  error('failed: missing month');
else
  fprintf('passed: no missing months\n');
end


for i = 1:size(aa,1)
  aa_new(i,:)          = -CISM_DX_FLAG*ones(1,248); % Make all months 31 days
  aa_new(i,1:8*Nd(i))  = aa(i,1:8*Nd(i));           % Fill in valid data
end

aa_new  = aa_new';   % Transpose
aa      = aa_new(:); % Flatten

% Remove non-days marked with -CISM_DX_FLAG.
I       = find(aa > -CISM_DX_FLAG + 1); 
aa      = aa(I);

% Put data on 1-hour time grid by repeating values
aa  = repmat(aa',3,1);

% Make time series a one column array.
aa  = aa(:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do some error checking
%
% Sanity check that the number of intervals matches what is expected.
L_diff  = 24*number_days(Year_o,Year_f)-length(aa);
if (L_diff != 0)
  error('failed: time series length is incorrect');
else
  fprintf('passed: time series length is correct\n');
end
% Next, check for off-by-one(day,year) errors by manually checking a few
% random data values

t             = 1+24*number_days(Year_o,1962); % First hour of 1963
check_diff(1) = aa(t)-8; % 8 was determined from data file aa1963.ima

if (check_diff(1) != 0)
  error('failed: first hour of 1963 test');
else
  fprintf('passed: first hour of 1963 test\n');
end

t             = 24*31 + 24*number_days(Year_o,1962); % Last hour of Jan 1963
check_diff(2) = aa(t)-12; % 12 was determined from data file aa1963.ima

if (check_diff(2) != 0)
  error('failed: last hour of Jan 1963 test');
else
  fprintf('passed: last hour of Jan 1963 test\n');
end

t             = 1 + 24*31 + 24*number_days(Year_o,2003); % First hour of Feb 2004
check_diff(3) = aa(t)-16; % 16 was determined from data file aa2004.ima

if (check_diff(3) != 0)
  error('failed: first hour of Feb 2004 test');
else
  fprintf('passed: first hour of Feb 2004 test\n');
end

% Plot sorted time series to make sure that there is a clear separation
% between flag values and actual data.
plot(sort(aa));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Save aa data in a (temporary) .mat file
com = sprintf('save %s%s aa Year_o Year_f',TEMP_DIR,TEMP_FILE);
      eval(com);


